# ChemicalIdentifierResolver.py - general component for StarDrop plugin scripts
#
# Copyright (C) Optibrium Ltd 2013
# stardrop-support@optibrium.com
#
# This script uses the web-based services provided by the 
# National Cancer Institute at http://cactus.nci.nih.gov/
# to look-up information about chemical structures

# For advice or feedback, contact stardrop-support@optibrium.com

from six.moves.urllib.request import urlopen

class ChemicalIdentifierResolver:
    def __init__( self ):
        self.url_start = 'http://cactus.nci.nih.gov/chemical/structure'

    def encodedSmiles( self, smiles ):
        encoded_smiles = smiles.replace( '%', '%25' )   
        encoded_smiles = encoded_smiles.replace( '(', '%28' )
        encoded_smiles = encoded_smiles.replace( ')', '%29' )
        encoded_smiles = encoded_smiles.replace( '=', '%3D' )
        encoded_smiles = encoded_smiles.replace( '[', '%5B' )
        encoded_smiles = encoded_smiles.replace( ']', '%5D' )
        encoded_smiles = encoded_smiles.replace( '#', '%23' )
        encoded_smiles = encoded_smiles.replace( '+', '%2B' )
        encoded_smiles = encoded_smiles.replace( '-', '%2D' )
        encoded_smiles = encoded_smiles.replace( '.', '%2E' )
        encoded_smiles = encoded_smiles.replace( '/', '%2F' )
        encoded_smiles = encoded_smiles.replace( '\\', '%5C' )
        encoded_smiles = encoded_smiles.replace( '@', '%40' )
        encoded_smiles = encoded_smiles.replace( ':', '%3A' )
        encoded_smiles = encoded_smiles.replace( '~', '%7E' )
        return encoded_smiles
    
    def IUPACName( self, smiles ):
        return self._resolve( smiles, 'iupac_name' )    
        
    def standardInChIKey( self, smiles ):
        key_string = str(self._resolve( smiles, 'stdinchikey' ))
        std_inchi = key_string.lstrip( 'InChIKey=' )
        return std_inchi
                
    def CASNumber( self, smiles ):
        return self._resolve( smiles, 'cas' )    
        
    def names( self, smiles ):   
        return self._resolve( smiles, 'names' )    
        
    def chemSpiderId( self, smiles ):
        return self._resolve( smiles, 'chemspider_id' )    
        
    def _resolve( self, smiles, convert_to ):
        encoded_smiles = self.encodedSmiles( smiles )
        url = self.url_start + '/' + encoded_smiles + '/' + convert_to
        resolver = urlopen( url )
        return resolver.read().decode()
    
        
def main( argv = None ):
    smiles = 'CC=O'
    resolver = ChemicalIdentifierResolver() 
    
    print ('IUPAC name is ', resolver.IUPACName( smiles ))
    
    print ('CAS number is ', resolver.CASNumber( smiles ))
    
    print ('Standard inchi key is ', resolver.standardInChIKey( smiles ))
    
    print ('ChemSpider ID is ', resolver.chemSpiderId( smiles ))
    
    print ('names are:\n', resolver.names( smiles ))
    
            
if __name__ == "__main__":
    main() 
    


       