# DataSetUtils.py - general component for StarDrop plugin scripts
#
# Copyright (C) Optibrium Ltd 2011
# stardrop-support@optibrium.com
#
# This script provides data set-related functions for StarDrop
# plugin scripts

import app
import PythonQt

import math

#parse the list of probabilities from a category entry
def catErrorString(entry):
    result = ""
    prob = 0.0
    for cat in entry.keys():
        if entry[cat] > prob:
            result = cat
            prob = entry[cat]
    return result,prob

def convertVal(val, u, du):
    rv = 0.0
    #do common cases first
    if u in (2,4,7) and du == u - 1:
        rv = math.pow(10,val)
    elif u in (1,3,6) and du == u + 1:
        rv = math.log10(val)
    elif u == 1 and du == 3:
        rv = val * 1000
    elif u == 3 and du == 1:
        rv = val/1000
    elif u == 1 and du == 4:
        rv = math.log10(val * 1000)
    elif u == 3 and du == 2:
        rv = math.log10(val/1000)
    elif u == 2 and du == 3:
        rv = math.pow(10,val) * 1000
    elif u == 2 and du == 4:
        rv = val + 3
    elif u == 4 and du == 1:
        rv = (math.pow(10,val))/1000
    elif u == 4 and du == 2:
        rv = val - 3
    elif u == 5:#pKi/IC50 values
        if du == 1:
            rv = math.pow(10, 6.0 - val)
        if du == 2:
            rv = 0.0 - val + 6.0
        if du == 3:
            rv = math.pow(10, 9.0 - val)
        if du == 4:
            rv = 0.0 - val + 9.0
    elif du == 5:#pK/IC50 display values
        if u == 1:
            rv = 0.0 - math.log10(val)
        if u == 2:
            rv = 0.0 - val + 6.0
        if u == 3:
            rv = 0.0 - math.log10(val * 1000)
        if u == 4:
            rv = 0.0 - val + 9.0
    return rv

#return the display value and error for a given entry as a tuple
def getDisplayValues(entry, coldata):
    if len(entry) == 0: #invalid entry
        return ("", "")
    val = ""
    stats = 0.0
    type = coldata[0]
    if type == app.regio:
        val = entry[0][2]
        stats = entry[0][3]
    elif type == app.score:
        val = entry[0][0]
        stats = entry[0][1]
    elif type == app.cont:
        #check here for different base type and display type
        if coldata[6][1][0] == coldata[6][1][1]:
            val = entry[1]
            stats = entry[2]
        else:
            val = convertVal(entry[1],coldata[6][1][0],coldata[6][1][1])
            stats = entry[2]
    elif type == app.cat:
        val,stats = catErrorString(entry[1])
    elif type == app.mol:
        val = entry[0]
    elif type == app.rgroup:
        val = entry[0]
    elif type == app.cresset:
        val = entry[0]
        stats = entry[1]
    elif type == app.datetime:
        val = entry[0]
    elif type == app.string:
        val_as_bytes = entry[0][:-1] # remove trailing null for string entries
        val = val_as_bytes.decode('utf-8')
        #print "Val:"
        #print val
        #print "Entry:"
        #print entry
    else:
         print ('Invalid column type')
    return (val,stats)

#parse a dataset and return a list of column names and types
def getColumnNames(data):
    ds = app.Dataset()
    ds.parse(data)
    names = {}
    for c in ds.columns:
        name = c[3][0]
        type = c[0]
        #check for score or regio
        #if type == 0 and c[1] in ("ScoreDataEntry", "RegioDataEntry"):
        #type = app.cont
        names[name] = type
    return names

#display a dialog showing a list of options, and return the selected one
def selectOption(options, title):
  dlg = PythonQt.PQDialog()
  dlg.title = title
  list = PythonQt.PQListWidget()
  for s in options:
    list.addItem(s)
  layout = PythonQt.PQVBoxLayout()
  layout.addWidget(list)
  dlg.setLayout(layout)
  if dlg.run() == True and len(list.selected) > 0:
    choice = str(list.selected[0])
  else:
    choice = ''
  return choice

#  A helper function to determine the first index of any structure column
#
#  Returns -1 if no structure column was found  
def structureColumnIndex( dataset ):
    index = 0
    for column in dataset.columns:
        if column[0] == 16:
            return index
        index = index + 1

    return -1		
	
# A helper function to extract a list of smiles from a data set
#
# The size of the list will be the same as the number of rows 
# in the data set	
def smilesList( dataset ):
    smiles_list = []
    structure_column_index = structureColumnIndex( dataset )
    for row in range(0,dataset.rowSz()):

        entry = dataset.get(structure_column_index,row)
        col_data = dataset.columnMeta( structure_column_index )
        try:
            (val,stats) = getDisplayValues(entry, col_data)
            smiles_list.append( val )
        except:
            (val,stats) = ('?', '?')
            smiles_list.append( val )
    return smiles_list		
        		

