# InChiKeys.py - plugin module for StarDrop
#
# Copyright (C) Optibrium Ltd 2013
# stardrop-support@optibrium.com
#
# This module enables the user to use the web service
# provided by National Cancer Institute (http://cactus.nci.nih.gov/)
# to attempt to generate an InChI key for a chemical structure

# For advice or feedback, contact stardrop-support@optibrium.com
import app
import math
import traceback
import PythonQt
from ChemicalIdentifierResolver import *
from DataSetUtils import *


def getInChIKey(dataset):
    # Open the current dataset
    ds = app.Dataset()
    ds.parse(dataset)
    smiles_list = smilesList(ds)
    if len(smiles_list) == 0:
        return 'No', False

    resolver = ChemicalIdentifierResolver()
    new_entries = []
    for smiles in smiles_list:
        try:
            inchi_key = str(resolver.standardInChIKey(smiles))
            new_entries.append(app.Dataset.string(inchi_key))
        except:
            print("Exception in getInChIKey")
            traceback.print_exc()
            new_entries.append(app.Dataset.string('unknown'))

    name_column = app.Dataset.meta('InChI key', app.string)
    structure_column_index = structureColumnIndex(ds)
    ds.insertColumn(name_column, new_entries, structure_column_index + 1)
    return (ds.packColumn(structure_column_index + 1), structure_column_index + 1), True


def getInChIKeyFromDataSet(stardrop_appui):
    dataset = stardrop_appui.importDS("")
    result = getInChIKey(dataset)
    if result[1] == False:
        if len(result[0]) > 0:
            stardrop_appui.show(result[0])
        else:
            stardrop_appui.show('InChIKey script failed')
    else:
        stardrop_appui.startMacro("")
        (data, index) = result[0]
        stardrop_appui.addColumnToDataSet('', index, data)
        stardrop_appui.stopMacro("")


def get_stardrop_definitions():
    definitions = [
        {
            'script_name': 'NIH web service: Generate InChI key',
            'callback': getInChIKeyFromDataSet
        }
    ]

    return definitions
